<?php
namespace TYPO3\CMS\Fluid\Core\Parser\SyntaxTree;

/*                                                                        *
 * This script is backported from the TYPO3 Flow package "TYPO3.Fluid".   *
 *                                                                        *
 * It is free software; you can redistribute it and/or modify it under    *
 * the terms of the GNU Lesser General Public License, either version 3   *
 *  of the License, or (at your option) any later version.                *
 *                                                                        *
 * The TYPO3 project - inspiring people to share!                         *
 *                                                                        */

use TYPO3\CMS\Fluid\Core\Parser\TemplateParser;

/**
 * Interface Syntax Tree Node - is a container for const defined in interface.
 */
class InterfaceNode extends TextNode
{
    /**
     * @var \TYPO3\CMS\Extbase\Reflection\ClassReflection
     */
    protected $reflection;

    /**
     * Contents of the interface
     *
     * @var string
     */
    protected $interface;

    /**
     * Contents of the parameter
     *
     * @var string
     */
    protected $parameter;

    /**
     * Constructor.
     *
     * @param string $text text to be stored in this node
     * @throws \TYPO3\CMS\Fluid\Core\Parser\Exception
     */
    public function __construct($text)
    {
        parent::__construct($text);

        preg_match(TemplateParser::$SCAN_PATTERN_SHORTHANDSYNTAX_INTERFACE, $text, $matches);

        if (!is_string($matches['Interface']) && !is_string($matches['PARAMETER'])) {
            throw new \TYPO3\CMS\Fluid\Core\Parser\Exception('Interface node requires the argument to be formatted with \\Vendor\Extension\\Interface\\SomeInterface::SOME_PARAMETER.');
        }

        if (interface_exists($matches['Interface'])) {
            $this->reflection = new \TYPO3\CMS\Extbase\Reflection\ClassReflection($matches['Interface']);
        } else {
            throw new \TYPO3\CMS\Fluid\Core\Parser\Exception('Interface ' . $matches['Interface'] . ' is not existed. ');
        }

        if (!in_array($matches['PARAMETER'], array_keys($this->reflection->getConstants()))) {
            throw new \TYPO3\CMS\Fluid\Core\Parser\Exception($matches['PARAMETER'] . ' is not found in ' . $matches['Interface'] . '.');
        }

        $this->interface = $matches['Interface'];
        $this->parameter = $matches['PARAMETER'];
    }

    /**
     * Return the parameter defined in the interface class.
     *
     * @param \TYPO3\CMS\Fluid\Core\Rendering\RenderingContextInterface $renderingContext
     * @return string the real parameter.
     */
    public function evaluate(\TYPO3\CMS\Fluid\Core\Rendering\RenderingContextInterface $renderingContext)
    {
        return $this->reflection->getConstant($this->parameter);
    }

    /**
     * Getter for interface class
     *
     * @return string The interface class of this node
     */
    public function getInterface()
    {
        return $this->interface;
    }

    /**
     * Getter for interface parameter
     *
     * @return string The interface parameter of this node
     */
    public function getParameter()
    {
        return $this->parameter;
    }
}
